<p class="auto-cursor-target">
  <br/>
</p>
<ac:structured-macro ac:macro-id="f7d319ca-89ea-400a-afc7-5454d72f14dc" ac:name="details" ac:schema-version="1">
  <ac:rich-text-body>
    <p class="auto-cursor-target">
      <br/>
    </p>
    <table>
      <colgroup>
        <col/>
        <col/>
      </colgroup>
      <tbody>
      </tbody>
    </table>
    <p class="auto-cursor-target">
      <br/>
    </p>
  </ac:rich-text-body>
</ac:structured-macro>
<p class="auto-cursor-target">
  <br/>
</p>
